﻿using Microsoft.Extensions.Logging;
using SewingDressesContracts.BindingModels;
using SewingDressesContracts.BusinessLogicsContracts;
using SewingDressesContracts.SearchModels;
using SewingDressesContracts.StoragesContracts;
using SewingDressesContracts.ViewModels;
using SewingDressesDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SewingDressesBusinessLogic.BusinessLogic
{
    public class ShopLogic : IShopLogic
    {
        private readonly ILogger _logger;
        private readonly IShopStorage _shopStorage;
        public ShopLogic(ILogger<ShopLogic> logger, IShopStorage shopStorage)
        {
            _logger = logger;
            _shopStorage = shopStorage;
        }
        public List<ShopViewModel> ReadList(ShopSearchModel model)
        {
            _logger.LogInformation("ReadList. ShopName:{Name}. Id:{ Id}", model?.Name, model?.Id);
            var list = model == null ? _shopStorage.GetFullList() : _shopStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public ShopViewModel ReadElement(ShopSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. ShopName:{ShopName}.Id:{ Id}", model.Name, model.Id);
            var element = _shopStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public bool Create(ShopBindingModel model)
        {
            CheckModel(model);
            if (_shopStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Update(ShopBindingModel model)
        {
            CheckModel(model);
            if (_shopStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(ShopBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_shopStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(ShopBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.ShopName))
            {
                throw new ArgumentNullException("Нет названия магазина",
               nameof(model.ShopName));
            }
            if (string.IsNullOrEmpty(model.Adress))
            {
                throw new ArgumentNullException("Нет адресса магазина",
               nameof(model.ShopName));
            }
            if (model.DateOpen == null)
            {
                throw new ArgumentNullException("Нет даты открытия магазина",
               nameof(model.ShopName));
            }
            _logger.LogInformation("Shop. ShopName:{ShopName}.Address:{Address}. DateOpen:{DateOpen}. Id: { Id}", model.ShopName, model.Adress, model.DateOpen, model.Id);
            var element = _shopStorage.GetElement(new ShopSearchModel
            {
                Name = model.ShopName
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Магазин с таким названием уже есть");
            }
        }

        public bool MakeSell(IDressModel model, int count)
        {
            return _shopStorage.SellDresses(model, count);
        }
        public bool MakeSupply(ShopSearchModel model, IDressModel dress, int count)
        {
            if (model == null)
                throw new ArgumentNullException(nameof(model));      
            if (dress == null)
                throw new ArgumentNullException(nameof(dress));
            if (count <= 0)
                throw new ArgumentNullException("Количество должно быть положительным числом");
            
            ShopViewModel? curModel = _shopStorage.GetElement(model);
            _logger.LogInformation("Make Supply. Id: {Id}. ShopName: {Name}",model.Id, model.Name);
            if (curModel == null)
                throw new ArgumentNullException(nameof(curModel));
            
            var countItems = curModel.ShopDresses.Select(x => x.Value.Item2).Sum();
            if (curModel.MaxCount - countItems < count)
            {
                _logger.LogWarning("Shop is overflowed");
                return false;
            }

            if (curModel.ShopDresses.TryGetValue(dress.Id, out var pair))
            {
                curModel.ShopDresses[dress.Id] = (pair.Item1, pair.Item2 + count);
                _logger.LogInformation("Make Supply. Add Dress. ShopName: {ShopName}. DressName: {Name}. Count: {Count}", curModel.ShopName, pair.Item1, count + pair.Item2);
            }
            else
            {
                curModel.ShopDresses.Add(dress.Id, (dress, count));
                _logger.LogInformation("Make Supply. Add new Dress. ShopName: {ShopName}. DressName: {Name}. Count: {Count}", curModel.ShopName, pair.Item1, count);
            }
           
            return Update(new()
            {
                Id = curModel.Id,
                ShopName = curModel.ShopName,
                DateOpen = curModel.DateOpen,
                MaxCount = curModel.MaxCount,
                Adress = curModel.Adress,
                ShopDresses = curModel.ShopDresses,
            });
        }
    }
}
